(function() {

    if (typeof Cc == "undefined") {
        var {Cc, Ci, Cu} = require("chrome");
     }
     
     const prefs = require("ko/prefs");
     const ajax = require("ko/ajax");
     const log = require("ko/logging").getLogger("Platform Auth");
     const legacy = require("ko/windows").getMain().ko;
     log.setLevel(require("ko/logging").LOG_INFO);
     // log.setLevel(require("ko/logging").LOG_DEBUG);

    var pwManager = Cc["@mozilla.org/login-manager;1"].getService(Ci.nsILoginManager);

    this.apiKeyUrlForPWM = "apikey.login.tld";
    this.userUrlForPWM = "user.login.tld";
    this.privkeyUrlForPWM = "privkey.login.tld";

    var apiUrl = prefs.getStringPref("platform.api.url");

    log.info("Using API URL: " + apiUrl);
    
    /* Callback should take a bool, True for success and False for failed authentication */
    this.authenticated = (callback) =>
    {
        let credentials = this.getCredentials(this.apiKeyUrlForPWM);
        if (credentials && this.getCredentials(this.userUrlForPWM) && this.getCredentials(this.privkeyUrlForPWM)) {
            this.loginWithKey(credentials.password, (code) => {
                // 0 means not connected to the internet
                if (code == "200" || code == "0") {
                    callback(true);
                } else {
                    this.removeSavedKeys();
                    callback(false);
                }
            });
        } else {
            callback(false);
        }
    };

    this.getCredentials = (url) =>
    {
        let pws = pwManager.findLogins({}, url, url, null);
        if (pws.length == 1)
        {
            return pws[0];
        } else
        {
            return;
        }
    };

    let getLoginDefaults = (url) =>
    {
        let loginInfo = Cc["@mozilla.org/login-manager/loginInfo;1"]
            .createInstance(Ci.nsILoginInfo);
        loginInfo.formSubmitURL = url;
        loginInfo.hostname = url;
        loginInfo.httpRealm = null;
        loginInfo.username = ""; // can't be null
        loginInfo.usernameField = ""; // can't be null
        loginInfo.passwordField = "";// can't be null
        return loginInfo;
    };

    this.saveUser = (username, password) => 
    {
        let loginInfo = getLoginDefaults(this.userUrlForPWM);
        let existing = this.getCredentials(this.userUrlForPWM);

        loginInfo.username = username;
        loginInfo.password = password;

        if(existing)
            pwManager.modifyLogin(existing, loginInfo);
        else
            pwManager.addLogin(loginInfo);
    };

    // This can be called to just
    this.saveSecret = (url, value) =>
    {
        let loginInfo = getLoginDefaults(url);
        
        loginInfo.password = value;
        let pw = this.getCredentials(url);
        
        if(pw) {
            pwManager.modifyLogin(pw, loginInfo);
        } else {
            pwManager.addLogin(loginInfo);
        }
    };

    this.removeSavedKeys = () =>
    {
        var urls = [this.apiKeyUrlForPWM, this.userUrlForPWM, this.privkeyUrlForPWM];
        for (let url of urls) 
        {
            let pws = pwManager.findLogins({}, url, url, null);
            if(pws)
            {
                for (var i = 0; i < pws.length; i++)
                {
                   pwManager.removeLogin(pws[i]);
                }
            }
        }
    };
    
    this.logout = () =>
    {
        this.loginAgain();
    };
    
    this.loginAgain = () =>
    {
        this.removeSavedKeys();
        this.loginDialog();
    };
   
    this.loginWithKey = (apikey, callerCallback) =>
    {
        this.login(JSON.stringify({"token":apikey}), callerCallback);
    };
   
    this.loginWithUser= (username, password, callerCallback, totp="") =>
    {
        this.login(JSON.stringify({password:password,username:username,totp:totp}), callerCallback);
    };
    
    this.loginDialog = () =>
    {
        var winOptions = 
        // #if PLATFORM == "darwin"
                "chrome,resizable=no,menubar,toolbar,status,all,dialog=no,centerscreen=yes,modal";
        // #else
                "chrome,resizable=no,menubar,toolbar,status,all,centerscreen=yes,modal";
        // #endif
        let returnObj = {}
        legacy.windowManager.openWindow(
            "chrome://komodo/content/login/login.xul",
            "_blank", winOptions, returnObj);
        return returnObj.OK;
    }

    // login and get the JWT token to perform the komodo auth step and API key step
    this.login = (body, callerCallback) =>
    {
        let callback = (code, response) =>
        {
            let jsonResp;
            try {
                jsonResp = JSON.parse(response);
            } catch (e) {
                log.error("Login failed with: " + e.message);
                log.error("Response: " + response, true);
            }

            if(code == "200" && jsonResp)
            {
                if( ! this.getCredentials(this.apiKeyUrlForPWM))
                {
                    this.authenticateKomodo(jsonResp.token, callerCallback);
                } else
                {
                    callerCallback(code);
                }
            }
            else if (jsonResp)
            {
                callerCallback(code, "Login failed with message: " + jsonResp.message + "  Please try again.");
            } 
            else if (code == "0")
            {
                callerCallback(code, "Login failed, it appears you are offline.");
            }
            else 
            {
                callerCallback(code, "Login failed due to an unknown reason. Please try again or contact support.");
            }
        };

        let params =
        {
            url : apiUrl+"/login",
            method : "POST",
            body : body,
            headers : {"Content-Type": "application/json"}
        };
        send(params, callback);
    };
   
   // Check they are allowed to use Komodo
   this.authenticateKomodo = (jToken, callerCallback) =>
   {
        let callback = (code) =>
        {
            if(code == "200")
            {
                this.getAPIKey(jToken, callerCallback);
            }
            else
            {
                callerCallback(code, "Not permitted to use Komodo.  Please upgrade your ActiveState Platform account or purchase the Komodo Addon.");
            } 
        };
        var params =
        {
            url : apiUrl+"/status/komodo",
            method : "GET",
            headers :
            {
                "Content-Type": "application/json",
                'Authorization': "Bearer "+jToken
            }
        };
        send(params, callback);
   };
   
   // get API token to save and use for subsequent logins
   this.getAPIKey = (jToken, callerCallback) =>
    {
        let callback = (code, response) =>
        {
            let jsonResp;
            try {
                jsonResp = JSON.parse(response);
            } catch (e) {
                log.error("apikeys request failed with: " + e.message);
                log.error("response: " + response, true);
            }
            if(code == "200" && jsonResp)
            {
                this.saveSecret(this.apiKeyUrlForPWM, jsonResp.token);
                callerCallback(code);
            }
            else if (jsonResp)
            {
                callerCallback(code, "Cannot complete login: " + jsonResp.message + ".  Please try again.");
            } 
            else 
            {
                callerCallback(code, "Login failed due to an unknown reason.  Please try again or contact support.");
            }
        };
        var params =
        {
            url : apiUrl+"/apikeys",
            method : "POST",
            body : JSON.stringify({name:"Komodo IDE Usage Token"}),
            headers :
            {
                "Content-Type": "application/json",
                'Authorization': "Bearer "+jToken
            }
        };
        send(params, callback);
   };

   var send = (params, callback) =>
   {
       ajax.request2(params, callback);
   };
    
}).apply(module.exports);